from django.conf import settings


def template_name(request):
    """Common settings to put in context."""
    template_path = getattr(settings,
                            'ACTIVITIES_BASE_TEMPLATE',
                            'activities/base_activities.html')
    return {
        'ACTIVITIES_BASE_TEMPLATE': template_path
    }



from __future__ import unicode_literals

from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _
from django_core.db.models.mixins.base import AbstractBaseModel
from django_core.db.models.mixins.generic import AbstractGenericObject
from django_core.db.models.mixins.urls import AbstractUrlLinkModelMixin

from .constants import Action
from .constants import Privacy
from .constants import Source
from .managers import ActivityManager
from .managers import ActivityReplyManager


class AbstractActivity(AbstractBaseModel):
    """Abstract extensible model for Activities.

    Attributes:

    * about: object the activity is referring to.
    * about_id: id this object the activity pertains to.
    * about_content_type: the content type of the about object
    * text: the text of the activity.  This can include html. This is not
        required because you can construct the text based on the other object
        attributes.  For example, if I have an object ``car`` with a status of
        ``CREATED`` then I can construct the text as follows:

            ``Car object`` was ``created`` by ``created_user``
    * replies: list of replies to this activity
    * for_objs: list of docs this activity is for. For example,
        if a comment is made on a object "A" which has an object "B", then this
        list will include references to the::

            1. object "A"
            2. object "B"
            4. users who are sharing this object

    * source: the source of activity.  Can be one if constant.Source
        choices (i.e. 'USER' generated comment, 'SYSTEM' activity on an
        object, etc)
    * action: the action (verb) that describes what happend ('CREATED',
        'UPDATED', 'DELETED', 'COMMENTED', etc)
    * privacy: the privacy of the activity.  Can be either 'PUBLIC' or
        'PRIVATE'.
    """
    text = models.TextField(blank=True, null=True)
    about = generic.GenericForeignKey(ct_field='about_content_type',
                                      fk_field='about_id')
    about_content_type = models.ForeignKey(ContentType, null=True, blank=True)
    about_id = models.PositiveIntegerField(null=True, blank=True)
    replies = models.ManyToManyField('ActivityReply',
                                     related_name='replies',
                                     blank=True)
    for_objs = models.ManyToManyField('ActivityFor',
                                      related_name='for_objs',
                                      blank=True)
    source = models.CharField(max_length=20, choices=Source.CHOICES)
    action = models.CharField(max_length=20, choices=Action.CHOICES)
    privacy = models.CharField(max_length=20, choices=Privacy.CHOICES,
                               default=Privacy.PUBLIC)
    objects = ActivityManager()

    class Meta:
        abstract = True

    def is_comment(self):
        """Boolean indicating if the activity type is a comment."""
        return self.action == Action.COMMENTED

    def is_activity(self):
        """Boolean indicating if the activity type is an activity."""
        return self.source == Source.SYSTEM

    def is_public(self):
        """Boolean indicating if the activity is public."""
        return self.privacy == Privacy.PUBLIC

    def add_reply(self, user, text, reply_to=None):
        """Adds a reply to a Activity

        :param user: the user the reply is from.
        :param text: the text for the reply.
        :param reply_to: is a reply to a specific reply.

        """
        reply = self.replies.create(created_user=user,
                                    last_modified_user=user,
                                    text=text,
                                    reply_to=reply_to,
                                    activity=self)
        # TODO: If the user isn't part of the for_objs they should be added
        #       because they are not part of the conversation?
        # self.activityfor_set.get_or_create_generic(content_object=user)
        return reply

    def get_text(self):
        """Gets the text for an object.  If text is None, this will construct
        the text based on the activity attributes.
        """
        if self.text:
            return self.text

        action = Action.get_display(self.action) or self.action

        if self.action == Action.COMMENTED:
            template = '{created_user} {action} on the {object_name} {object}'
        else:
            template = '{created_user} {action} the {object_name} {object}'

        # TODO: need to differentiate system activity from user created
        #       activity.
        return template.format(
            created_user=self.created_user.username,
            action=action.lower(),
            object_name=self.about_content_type.model_class()._meta.verbose_name,
            object=self.about
        )

    def get_html(self):
        """Does the same thing as ``get_text(...)`` but looks to see
        if the objects have the ``get_absolute_url`` method implemented.  If
        they do, then they will appear as links.  For example, if the user
        model implements the ``get_absolute_url`` the user's text will be
        hyperlinked to the user's absolute url.
        """
        if self.text:
            return self.text

        # check to see if the about model has implemented a custom template
        if self.action == Action.CREATED and \
           hasattr(self.about, 'get_activity_created_html'):
            return self.about.get_activity_created_html(self)

        action = Action.get_display(self.action) or self.action

        created_user = self.created_user

        if hasattr(self.created_user, 'get_absolute_url_link'):
            created_user = self.created_user.get_absolute_url_link()
        elif hasattr(self.created_user, 'get_absolute_url'):
            created_user = '<a href="{0}">{1}</a>'.format(
                created_user.get_absolute_url(),
                created_user.username
            )
        else:
            created_user = self.created_user.username

        about = self.about

        if hasattr(self.about, 'get_absolute_url_link'):
            about = self.about.get_absolute_url_link()
        elif hasattr(self.about, 'get_absolute_url'):
            about = '<a href="{0}">{1}</a>'.format(about.get_absolute_url(),
                                                   about)

        if self.action == Action.COMMENTED:
            template = _('{created_user} {action} on the {object_name} '
                         '{object}.')
        else:
            template = _('{created_user} {action} the {object_name} {object}.')

        # TODO: need to differentiate system activity from user created
        #       activity.
        return template.format(
            created_user=created_user,
            action=action.lower(),
            object_name=self.about_content_type.model_class()._meta.verbose_name,
            object=about
        )

    def get_for_objects(self):
        """Gets the actual objects the activity is for."""
        return [obj.content_object
                for obj in self.for_objs.all().prefetch_related(
                                                            'content_object')]

    def get_replies(self):
        """Gets the activity reply objects for this activity."""
        return self.activityreply_set.all()

    def get_reply_by_id(self, reply_id):
        """Gets the reply for a activity by it's id."""
        return self.activityreply_set.get(id=reply_id)

    def delete_reply(self, reply_id):
        """Delete an individual activity reply.

        :param activity_id: ID of the activity reply to delete

        """
        self.activityreply_set.filter(id=reply_id).delete()
        return True


class Activity(AbstractUrlLinkModelMixin, AbstractActivity):
    """Concrete model for activities."""

    objects = ActivityManager()

    class Meta:
        ordering = ('-id',)
        index_together = (('about_content_type', 'about_id'),)

    def get_absolute_url(self):
        return '/activities/{0}'.format(self.id)

    def get_edit_url(self):
        return '{0}/edit'.format(self.get_absolute_url())

    def get_delete_url(self):
        return '{0}/delete'.format(self.get_absolute_url())


class ActivityReply(AbstractUrlLinkModelMixin, AbstractBaseModel):
    """Represents a activity reply object.

    Attributes:

    * text: the text of the activity.  This can include html.
    * created_user: the person the activity was from.  This is the user who
            caused the activity to change.  This can be the same user as
            the activity is intended for (users can create activities
            for themselves)
    * reply_to: this is a reply to a reply and is a recursive reference.

    """
    text = models.TextField(max_length=500)
    activity = models.ForeignKey('Activity')
    reply_to = models.ForeignKey('self', blank=True, null=True)
    objects = ActivityReplyManager()

    class Meta:
        ordering = ('id',)

    def get_absolute_url(self):
        return '{0}/replies/{1}'.format(self.activity.get_absolute_url(),
                                        self.id)

    def get_edit_url(self):
        return '{0}/edit'.format(self.get_absolute_url())

    def get_delete_url(self):
        return '{0}/delete'.format(self.get_absolute_url())


@python_2_unicode_compatible
class ActivityFor(AbstractGenericObject):
    """Defines the generic object a activity is for."""

    def __str__(self):
        return '{0} {1}'.format(self.content_type, self.object_id)



from __future__ import unicode_literals

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import TemplateView
from django.views.generic.edit import DeleteView
from django.views.generic.edit import FormView
from django.views.generic.edit import UpdateView
from django_core.views import PagingViewMixin
from django_core.views.mixins.generic import GenericObjectViewMixin

from .forms import ActivityDeleteForm
from .forms import ActivityEditForm
from .forms import ActivityReplyEditForm
from .mixins.views import ActivitiesViewMixin
from .mixins.views import ActivityCreatedUserRequiredViewMixin
from .mixins.views import ActivityFormView
from .mixins.views import ActivityReplySingleObjectViewMixin
from .mixins.views import ActivitySingleObjectViewMixin
from .mixins.views import UserActivitiesViewMixin


class ActivitiesView(PagingViewMixin, ActivitiesViewMixin, ActivityFormView):

    template_name = 'activities/view_activities.html'


class ActivitiesGenericObjectView(GenericObjectViewMixin, ActivitiesView):

    def get_activities_about_object(self):
        return self.content_object

    # TODO: this would no longer work for the current configuration
    def get_content_object_url(self):
        url_args = [self.generic_object_content_type.id,
                    self.content_object.id]
        return reverse('activities_view', args=url_args)


# TODO: Is this view app specific?
class ActivitiesForUserView(PagingViewMixin, UserActivitiesViewMixin,
                            ActivityFormView):

    template_name = 'activities/view_activities.html'


class ActivityView(ActivitySingleObjectViewMixin, TemplateView):

    template_name = 'activities/view_activity.html'


class ActivityEditView(ActivityCreatedUserRequiredViewMixin,
                       ActivitySingleObjectViewMixin, UpdateView):
    template_name = 'activities/edit_activity.html'
    form_class = ActivityEditForm

    def get_form_kwargs(self):
        kwargs = super(ActivityEditView, self).get_form_kwargs()
        kwargs['instance'] = self.activity
        kwargs['user'] = self.request.user
        return kwargs

    def get_success_url(self):
        return self.activity.get_absolute_url()


class ActivityDeleteView(ActivityCreatedUserRequiredViewMixin,
                         ActivitySingleObjectViewMixin, FormView):

    template_name = 'activities/delete_activity.html'
    form_class = ActivityDeleteForm

    def form_valid(self, form):

        if self.activity.is_comment() and \
           self.activity.created_user == self.request.user:
            # The user created this comment.  They can delete it for all.
            self.activity.delete()
        else:
            # Isn't this users comment, but they don't want this comment
            # showing up in the feed.  Remove it.
            self.activity.for_objs.remove(self.request.user)

        if self.request.is_ajax():
            return HttpResponse('success', status=200)

        return super(ActivityDeleteView, self).form_valid(form)

    def get_success_url(self):
        return self.activity.about.get_absolute_url()


class ActivityRepliesView(ActivitySingleObjectViewMixin, TemplateView):
    template_name = 'activities/view_activity_replies.html'


class ActivityReplyView(ActivityReplySingleObjectViewMixin, TemplateView):
    template_name = 'activities/view_activity_reply.html'


class ActivityReplyEditView(ActivityCreatedUserRequiredViewMixin,
                            ActivityReplySingleObjectViewMixin, UpdateView):
    template_name = 'activities/edit_activity_reply.html'
    form_class = ActivityReplyEditForm

    def get_form_kwargs(self):
        kwargs = super(ActivityReplyEditView, self).get_form_kwargs()
        kwargs['instance'] = self.activity_reply
        kwargs['user'] = self.request.user
        return kwargs

    def form_valid(self, form):
        messages.success(request=self.request,
                         message=_('Reply successfully updated!'),
                         fail_silently=True)
        return super(ActivityReplyEditView, self).form_valid(form)

    def get_success_url(self):
        return self.request.path


class ActivityReplyDeleteView(ActivityCreatedUserRequiredViewMixin,
                              ActivityReplySingleObjectViewMixin, DeleteView):

    template_name = 'activities/delete_activity_reply.html'

    def get_success_url(self):
        return self.activity.about.get_absolute_url()



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.core.paginator import EmptyPage
from django.core.paginator import Paginator
from django.http.response import Http404
from django.http.response import HttpResponse
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.edit import FormView
from django_core.views.mixins.auth import LoginRequiredViewMixin

from .. import get_activity_model
from ..constants import Action
from ..constants import Source
from ..forms import BasicCommentForm
from ..http import ActivityResponse
from ..models import ActivityReply


Activity = get_activity_model()


class ActivityViewMixin(object):
    """View mixin for a single activity object."""

    activity = None
    activity_pk_url_kwarg = 'activity_id'

    def dispatch(self, *args, **kwargs):
        self.activity = self.get_activity(**kwargs)
        return super(ActivityViewMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ActivityViewMixin, self).get_context_data(**kwargs)
        context['activity'] = self.activity
        context['activity_url'] = self.get_activity_url()
        return context

    def get_activity(self, **kwargs):
        if self.activity:
            return self.activity

        self.activity = Activity.objects.get_by_id_or_404(
            id=kwargs.get(self.activity_pk_url_kwarg)
        )

        if self.activity.is_public() or \
           self.activity.created_user == self.request.user:
            return self.activity

        # Check to ensure the user has permission to view the activity
        user_ct = ContentType.objects.get_for_model(self.request.user)
        if not self.activity.for_objs.filter(content_type=user_ct,
                                             object_id=self.request.user.id):
            raise Http404

        return self.activity

    def get_activity_url(self):
        """Gets the root activity url for the object the activity is about."""
        prefix = ''

        if hasattr(self.activity.about, 'get_absolute_url'):
            prefix = self.activity.about.get_absolute_url()

        return '{0}/activities'.format(prefix)


class ActivitySingleObjectViewMixin(ActivityViewMixin, SingleObjectMixin):
    """Mixin for when the activity represents what the page is about."""

    def dispatch(self, *args, **kwargs):
        self.object = self.get_object(**kwargs)
        return super(ActivitySingleObjectViewMixin, self).dispatch(*args,
                                                                   **kwargs)

    def get_object(self, **kwargs):
        return self.get_activity(**kwargs)


class ActivityReplyViewMixin(ActivityViewMixin):
    """View mixin for a activity reply."""

    activity_reply = None
    activity_reply_pk_url_kwarg = 'reply_id'

    def dispatch(self, *args, **kwargs):
        self.activity_reply = self.get_activity_reply(**kwargs)
        # I do this do the correct activity proxy model is used.
        self.activity_reply.activity = self.get_activity(**kwargs)
        return super(ActivityReplyViewMixin,
                     self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ActivityReplyViewMixin,
                        self).get_context_data(**kwargs)
        context['activity_reply'] = self.activity_reply
        return context

    def get_activity_reply(self, **kwargs):
        if self.activity_reply:
            return self.activity_reply

        self.activity_reply = ActivityReply.objects.get_by_id_or_404(
            id=kwargs.get(self.activity_reply_pk_url_kwarg),
            select_related=True
        )
        return self.activity_reply


class ActivityReplySingleObjectViewMixin(ActivityReplyViewMixin,
                                         SingleObjectMixin):
    """Mixin for when the activity reply represents what the page is about.
    """

    def dispatch(self, *args, **kwargs):
        self.object = self.get_object(**kwargs)
        return super(ActivityReplySingleObjectViewMixin,
                     self).dispatch(*args, **kwargs)

    def get_object(self, **kwargs):
        return self.get_activity_reply(**kwargs)


class ActivitiesViewMixin(object):
    """Activities view mixin that returns the activity_obj's paginator
    and current page the the authenticated user is able to see.

    Filtering:

    You can further filter the activities by passing the following query
    string params:

    * ns: activity source.  Can be one of .contants.Source.

    Note: This mixin requires the django_core.mixins.paging.PagingViewMixin
    to be called before this view is called.
    """
    activities_page_num = 1
    activities_page_size = 15
    activities_paginate_by = activities_page_size
    activities_page_kwarg = 'np'
    activities_page_size_kwarg = 'nps'

    def dispatch(self, *args, **kwargs):

        if self.activities_page_size != self.activities_paginate_by:
            # Default to the paginate_by value if these two if different.
            self.activities_page_size = self.activities_paginate_by

        self.activities_page_num, self.activities_page_size = \
            self.get_activities_paging()
        self.activities_paginate_by = self.activities_page_size
        return super(ActivitiesViewMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ActivitiesViewMixin,
                        self).get_context_data(**kwargs)

        activities = self.get_activities_queryset()

        paginator = Paginator(activities, self.activities_page_size)
        context['activities_paginator'] = paginator

        try:
            context['activities_page'] = \
                paginator.page(self.activities_page_num)
        except EmptyPage:
            context['activities_page'] = paginator.page(paginator.num_pages)

        context['activities_about_object'] = \
            self.get_activities_about_object()

        if 'activity_url' not in context:
            context['activity_url'] = self.get_activity_url()

        return context

    def get_activities_common_queryset(self, queryset):
        """Common filters to apply to a queryset."""
        activity_kwargs = {}
        activity_source = Source.check(self.request.GET.get('ns'))

        if activity_source:
            activity_kwargs['source'] = activity_source

        activity_action = Action.check(self.request.GET.get('na'))

        if activity_action:
            activity_kwargs['action'] = activity_action

        return (queryset.filter(**activity_kwargs)
                        .prefetch_related('about',
                                          'about_content_type',
                                          'replies',
                                          'replies__created_user',
                                          'created_user'))

    def get_activities_queryset(self):
        """Get's the queryset for the activities."""
        activities_about_object = self.get_activities_about_object()
        activity_kwargs = {'for_user': self.request.user}

        if activities_about_object:
            activity_kwargs['obj'] = activities_about_object

        queryset = Activity.objects.get_for_object(**activity_kwargs)
        return self.get_activities_common_queryset(queryset=queryset)

    def get_activities_about_object(self):
        """Gets the object to get activities for. The default is to return
        self.object which will be set if using a DetailView.  Otherwise, the
        consuming View can override this method.

        This can be a single object or a list, tuple or set of objects.
        """
        if hasattr(self, 'object') and self.object is not None:
            return self.object

        return self.get_object()

    def get_activity_url(self):
        """Get the activity url for the activity object."""
        # TODO: Would be nice to try and optimize this so I don't have to query
        #       for the content type if possible.
        about_obj = self.get_activities_about_object()
        # content_type = ContentType.objects.get_for_model(about_obj)
        # return reverse('activities_view', args=[content_type.id, about_obj.id])

        prefix = ''

        if hasattr(about_obj, 'get_absolute_url'):
            prefix = about_obj.get_absolute_url()

        return '{0}/activities'.format(prefix)

    def get_activities_paging(self):
        """Gets the paging values passed through the query string params.

            * "np" for "activities page number" and
            * "nps" for "activities page size".

        :returns: tuple with the page being the first part and the page size
            being the second part.
        """
        orig_page_num = self.activities_page_num
        orig_page_size = self.activities_page_size

        try:

            page_num = int(self.request.GET.get(self.activities_page_kwarg))

            if page_num < 1:
                page_num = orig_page_num
        except:
            page_num = orig_page_num

        try:
            orig_page_size = self.activities_page_size
            page_size = int(self.request.GET.get(
                self.activities_page_size_kwarg)
            )

            if page_size < 1:
                page_size = orig_page_size
        except:
            page_size = orig_page_size

        return page_num, page_size


class ActivityCreatedUserRequiredViewMixin(LoginRequiredViewMixin):
    """View mixin for activity views that require the created user."""
    def dispatch(self, *args, **kwargs):

        if self.request.user != self.get_object(**kwargs).created_user:
            raise PermissionDenied

        return super(ActivityCreatedUserRequiredViewMixin,
                     self).dispatch(*args, **kwargs)


class UserActivitiesViewMixin(ActivitiesViewMixin):
    """Activities for the authenticated user."""

    def get_activities_about_object(self):
        return self.request.user


class ActivityFormView(FormView):
    """This is a form view that handles adding and displaying activities
    for an object.

    This must be called after after any subclass of
    activities.mixins.ActivitiesViewMixin so this mixin has access
    to the `activities_about_object` attribute.
    """
    form_class = BasicCommentForm

    def get(self, request, *args, **kwargs):
        # TODO: do I need to make an additional check here to make sure this is
        #       an ajax get request for activities?
        if self.request.is_ajax():
            return render_to_response(
                'activities/snippets/activities.html',
                self.get_context_data(),
                context_instance=RequestContext(self.request)
            )

        return super(ActivityFormView, self).get(request=request,
                                                 *args,
                                                 **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ActivityFormView, self).get_context_data(**kwargs)

        if self.request.is_ajax():
            context['obj'] = self.get_activities_about_object()

        return context

    def form_valid(self, form):

        # TODO: Need to make sure the user has the ability to comment on this
        #       object (they are sharing or authorized to do so.  Also,
        #       probably want to require ajax here?
        text = form.cleaned_data.get('text').strip()

        # if "pid" (parent_activity_id) exists, then this is a reply to a
        # activity.
        parent_activity_id = form.cleaned_data.get('pid')

        if parent_activity_id:
            # reply to an existing activity
            activity = Activity.objects.get_by_id(
                id=parent_activity_id
            )

            if not activity:
                raise HttpResponse(status=400)

            activity.add_reply(user=self.request.user, text=text)

        else:
            ensure_for_objs = []
            about = self.get_activities_about_object()
            if about == self.request.user:
                # user commenting on own wall
                ensure_for_objs.append(self.request.user)

            # New activity for an object
            activity = Activity.objects.create(
                created_user=self.request.user,
                text=text,
                about=about,
                source=Source.USER,
                action=Action.COMMENTED,
                ensure_for_objs=ensure_for_objs or None
            )

        if self.request.is_ajax():
            return ActivityResponse(request=self.request, activity=activity)

        # TODO: Where do I redirect to if it's not an ajax request?
        # return ''  # safe_redirect(self.request.POST.get('next') or '/')
        return super(ActivityFormView, self).form_valid(form=form)

    def form_invalid(self, form):
        # TODO: what do I want to do here?
        return super(ActivityFormView, self).form_invalid(form=form)



from __future__ import unicode_literals

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django_core.forms.fields import CharFieldStripped
from django_core.forms.mixins.users import UserFormMixin
from django_core.forms.widgets import ReadonlyWidget

from .models import Activity
from .models import ActivityReply


class BasicCommentForm(forms.Form):
    """Basic form for commenting."""

    text = CharFieldStripped(max_length=500, widget=forms.Textarea())
    # Parent id of the comment or activity
    pid = forms.IntegerField(required=False, widget=forms.HiddenInput())
    next = CharFieldStripped(max_length=999999, required=False)


class BaseActivityEditForm(UserFormMixin, forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(BaseActivityEditForm, self).__init__(*args, **kwargs)

        self.fields['created_user'].label = 'Created by'
        self.fields['created_user'].widget.attrs['value'] = \
            self.instance.created_user.get_full_name()
        self.fields['created_dttm'].label = 'Created'
        self.fields['created_dttm'].widget.attrs['readonly'] = 'readonly'
        self.fields['last_modified_user'].label = 'Last Modified by'
        self.fields['last_modified_user'].widget.attrs['value'] = \
            self.instance.last_modified_user.get_full_name()
        self.fields['last_modified_dttm'].label = 'Last Modified'
        self.fields['last_modified_dttm'].widget.attrs['readonly'] = 'readonly'

    def clean_created_dttm(self):
        return self.instance.created_dttm

    def clean_created_user(self):
        return self.instance.created_user

    def clean_last_modified_user(self):

        if not self.user.is_authenticated():
            raise ValidationError(_('User must be logged in.'))

        return self.user

    def clean(self, *args, **kwargs):
        cleaned_data = super(BaseActivityEditForm, self).clean(*args, **kwargs)

        # created user shouldn't error since it should always be the same
        if 'created_user' in self.errors:
            cleaned_data['created_user'] = self.instance.created_user
            del self.errors['created_user']

        if self.user.is_authenticated() and 'last_modified_user' in self.errors:
            cleaned_data['last_modified_user'] = self.user
            del self.errors['last_modified_user']

        return cleaned_data


class ActivityEditForm(BaseActivityEditForm):

    class Meta:
        model = Activity
        fields = ('created_user', 'created_dttm', 'last_modified_user',
                  'last_modified_dttm', 'text')
        widgets = {
            'created_user': ReadonlyWidget(),
            'last_modified_user': ReadonlyWidget()
        }


class ActivityReplyEditForm(BaseActivityEditForm):

    class Meta:
        model = ActivityReply
        fields = ('created_user', 'created_dttm', 'last_modified_user',
                  'last_modified_dttm', 'text')
        widgets = {
            'created_user': ReadonlyWidget(),
            'last_modified_user': ReadonlyWidget()
        }


class ActivityDeleteForm(forms.Form):
    """Form for deleting a activity."""



from __future__ import unicode_literals

from django.template import Library
from django.template.loader import render_to_string
from django_core.utils.loading import get_function_from_settings


register = Library()


@register.simple_tag(takes_context=True)
def render_activities(context, page, obj, activity_url, activity_source=None):
    """Renders an activities.

    :param page: the django page object of activities
    :param obj: the obj the activities are about
    :param activity_url: the url to use for the activities
    :param source: the activity source
    """
    template_context = {
        'activities_page': page,
        'obj': obj,
        'activity_url': activity_url
    }

    if activity_source is not None:
        template_context['activity_source'] = activity_source

    return render_to_string('activities/snippets/activities.html',
                            template_context,
                            context_instance=context)


@register.simple_tag(takes_context=True)
def render_activity(context, activity, activity_url, show_reference_obj=False):
    template_context = {
        'activity': activity,
        'show_reference_obj': show_reference_obj,
        'activity_url': activity_url
    }

    return render_to_string('activities/snippets/activity.html',
                            template_context,
                            context_instance=context)


@register.filter
def render_activity_form(form):
    """Renders a form based on settings ``ACTIVITIES_FORM_RENDERER``
    setting.  This allows users to plug in different 3rd party form rendering
    apps while being able to maintain a consistent look and feel across their
    site.

    The function must accept 1 argument which is the form to be rendered. No
    other args will be passed.

    For example,  if I want to use the
    `django-bootstrap-form <https://github.com/tzangms/django-bootstrap-form>`_
    app to render forms, I would provide the following setting to the template
    tag form rendering function::

        ACTIVITIES_FORM_RENDERER = 'bootstrapform.templatetags.bootstrap.bootstrap'

    Then all forms will render using the django-bootstrap-form library.  You
    can optionally provide the following strings that will render that form
    using table, paragraph or list tags::

        ACTIVITIES_FORM_RENDERER = 'as_p'     # render form using <p> tags
        ACTIVITIES_FORM_RENDERER = 'as_table' # render form using <table>
        ACTIVITIES_FORM_RENDERER = 'as_ul'    # render form using <ul>

    This will default to rending the form to however the form's ``__str__``
    method is defined.
    """
    renderer_func = get_function_from_settings('ACTIVITIES_FORM_RENDERER')
    if not renderer_func:
        return form

    if renderer_func == 'as_table':
        return form.as_table()

    if renderer_func == 'as_ul':
        return form.as_ul()

    if renderer_func == 'as_p':
        return form.as_p()

    return renderer_func(form)



from __future__ import unicode_literals

from django.template.context import RequestContext
from django.template.loader import render_to_string

try:
    # python 2
    import urlparse as parse
except ImportError:
    # python 3
    from urllib import parse


def get_activity_html(request, activity):
    """Gets the html for a activity.

    :param activity: activity document
    """
    context = {'activity': activity}

    # TODO: Need to fix all the url logic checking below since the
    #       absolute url != to the http referrer.
#     try:
#         about_obj_absolute_url = activity.about.get_absolute_url()
#         activity_url = '{0}/activities'.format(about_obj_absolute_url)
#     except:
#         about_obj_absolute_url = None
#         activity_url = None
#
#     current_url = parse.urlsplit(request.META.get('HTTP_REFERER', '')).path
#
#     if (about_obj_absolute_url != current_url and
#         activity_url not in current_url):
#         context['show_reference_obj'] = True

    return render_to_string('activities/snippets/activity.html',
                            context,
                            context_instance=RequestContext(request)).strip()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('created_dttm', models.DateTimeField(default=datetime.datetime.utcnow)),
                ('last_modified_dttm', models.DateTimeField(default=datetime.datetime.utcnow)),
                ('text', models.TextField(blank=True, null=True)),
                ('about_id', models.PositiveIntegerField(blank=True, null=True)),
                ('source', models.CharField(max_length=20, choices=[('SYSTEM', 'System'), ('USER', 'User')])),
                ('action', models.CharField(max_length=20, choices=[('COMMENTED', 'Commented'), ('CREATED', 'Created'), ('DELETED', 'Deleted'), ('UPDATED', 'Updated')])),
                ('privacy', models.CharField(max_length=20, choices=[('PUBLIC', 'Public - everyone can see'), ('PRIVATE', 'Private - only created user can see'), ('CUSTOM', 'Custom - users must be granted visibility')], default='PUBLIC')),
                ('about_content_type', models.ForeignKey(to='contenttypes.ContentType', null=True, blank=True)),
                ('created_user', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='activities_activity_created_user+')),
            ],
            options={
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='ActivityFor',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(db_index=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityReply',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('created_dttm', models.DateTimeField(default=datetime.datetime.utcnow)),
                ('last_modified_dttm', models.DateTimeField(default=datetime.datetime.utcnow)),
                ('text', models.TextField(max_length=500)),
                ('activity', models.ForeignKey(to='activities.Activity')),
                ('created_user', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='activities_activityreply_created_user+')),
                ('last_modified_user', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='activities_activityreply_last_modified_user+')),
                ('reply_to', models.ForeignKey(to='activities.ActivityReply', null=True, blank=True)),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.AddField(
            model_name='activity',
            name='for_objs',
            field=models.ManyToManyField(to='activities.ActivityFor', blank=True, related_name='for_objs'),
        ),
        migrations.AddField(
            model_name='activity',
            name='last_modified_user',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='activities_activity_last_modified_user+'),
        ),
        migrations.AddField(
            model_name='activity',
            name='replies',
            field=models.ManyToManyField(to='activities.ActivityReply', blank=True, related_name='replies'),
        ),
        migrations.AlterIndexTogether(
            name='activity',
            index_together=set([('about_content_type', 'about_id')]),
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import patterns
from django.conf.urls import url
from django.core.exceptions import ImproperlyConfigured

from .views import ActivitiesGenericObjectView
from .views import ActivitiesView
from .views import ActivityDeleteView
from .views import ActivityEditView
from .views import ActivityRepliesView
from .views import ActivityReplyDeleteView
from .views import ActivityReplyEditView
from .views import ActivityReplyView
from .views import ActivityView


# need to map these in order for the dynamic urls to work correctly
urlpattern_mapping = (
    {'regex': r'^/?$', 'view': ActivitiesView, 'name': 'activities_view'},
    {'regex': r'^/(?P<activity_id>\d+)/delete/?$', 'view': ActivityDeleteView, 'name': 'activity_delete'},
    {'regex': r'^/(?P<activity_id>\d+)/edit/?$', 'view': ActivityEditView, 'name': 'activity_edit'},
    {'regex': r'^/(?P<activity_id>\d+)/replies/(?P<reply_id>\d+)/delete/?$', 'view': ActivityReplyDeleteView, 'name': 'activity_reply_delete'},
    {'regex': r'^/(?P<activity_id>\d+)/replies/(?P<reply_id>\d+)/edit/?$', 'view': ActivityReplyEditView, 'name': 'activity_reply_edit'},
    {'regex': r'^/(?P<activity_id>\d+)/replies/(?P<reply_id>\d+)/?$', 'view': ActivityReplyView, 'name': 'activity_reply'},
    {'regex': r'^/(?P<activity_id>\d+)/replies/?$', 'view': ActivityRepliesView, 'name': 'activity_replies'},
    {'regex': r'^/(?P<activity_id>\d+)/?$', 'view': ActivityView, 'name': 'activity_view'},
)


urlpatterns = patterns('',
    url(r'^/(?P<content_type_id>\d+)/(?P<object_id>\d+)/?$', ActivitiesGenericObjectView.as_view(), name='activities_view'),
    *[url(pattern['regex'], pattern['view'].as_view(), name=pattern['name'])
      for pattern in urlpattern_mapping if pattern['name'] != 'activities_view']
)


def get_urls(extend_urlpatterns, root_urlpattern_name, class_prefix,
             bases_classes=None):
    """Function that dynamically creates activities urls so urls don't have to
    use generic content type ids in the urls.

    :param extend_urlpatterns: the urls patterns to extend
    :param root_urlpattern_name: this is the url pattern to extend the activity
        urls from.
    :param class_prefix: this is the string class prefix to use for the
        generated views.  This will also be the prefix used for the url naming
        conventions.
    :param bases: the iterable of base classes to extend.
    :param url_prefix: the prefix to use for the urls.  Default is "activities"
        so the urls would be generated as follows:

        ./activities/(?P<activity_id>\d+)/delete/?S
        ./activities/(?P<activity_id>\d+)/edit/?S

    A call to the following:

    > get_urls(my_url_patterns,
    .          class_prefix='Foo',
    .          base_classes=(LoginRequiredViewMixin,))

    This generates a view class that resembles something along the lines of:

    class FooActivityView(LoginRequiredViewMixin, ActivityView):
        class_prefix = 'Foo'

    """
    root_urlpattern = None

    for pattern in extend_urlpatterns:
        if pattern.name == root_urlpattern_name:
            root_urlpattern = pattern.regex.pattern
            break

    if root_urlpattern is None:
        raise ImproperlyConfigured('No url pattern found with the name: '
                                   '{0}'.format(root_urlpattern_name))

    # root url pattern can't end in any of these characters
    for char in ('$', '?', '/'):
        if root_urlpattern.endswith(char):
            root_urlpattern = root_urlpattern[:-1]

    for pattern in urlpattern_mapping:
        # generate the dynamic view
        pattern_view = pattern.get('view')
        pattern_name = pattern.get('name')
        pattern_regex = pattern.get('regex').replace('^/', '')

        class_name = '{0}{1}'.format(class_prefix, pattern_view.__name__)
        ExtendedActivityView = type(class_name,
                                    bases_classes + (pattern_view,),
                                    {'class_prefix': class_prefix})

        # generate the new pattern name
        pattern_name = '{0}_{1}'.format(class_prefix.lower(), pattern_name)
        url_pattern = r'{0}/activities/{1}'.format(root_urlpattern,
                                                   pattern_regex)

        # add the pattern to urls
        extend_urlpatterns += patterns('',
            url(url_pattern, ExtendedActivityView.as_view(), name=pattern_name),
        )



from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured


def get_activity_model():
    """Return the Activity model that is active in this project.

    This is the same pattern user for django's "get_user_model()" method. To
    allow you to set the model instance to a different model subclass.
    """
    from django.conf import settings
    from django.db.models import get_model

    if not hasattr(settings, 'ACTIVITY_MODEL'):
        from .models import Activity
        return Activity

    try:
        app_label, model_name = settings.ACTIVITY_MODEL.split('.')
    except ValueError:
        raise ImproperlyConfigured("ACTIVITY_MODEL must be of the form "
                                   "'app_label.model_name'")

    activity_model = get_model(app_label, model_name)

    if activity_model is None:
        raise ImproperlyConfigured("ACTIVITY_MODEL refers to model '%s' "
                                   "that has not been installed" %
                                   settings.ACTIVITY_MODEL)

    return activity_model



from __future__ import unicode_literals

from django_core.views import JSONResponse

from .utils import get_activity_html


class ActivityResponse(JSONResponse):
    """Returns a HttpResponse for an event that causes a new activity to be
    generated.  The content of the response is the new activity. Returns a
    status of 200.

    Response content sample:

    {
        activity: "activity html",
        additional_content_key1: additional_content_value1
    }

    :param activity: activity doc
    :param additional_content: a dictionary of additional content that should
        be returned with the response.
    :return: HttpResponse with json encoded activity content.
    """
    def __init__(self, request, activity, status=200, additional_content=None,
                 **kwargs):

        if additional_content:
            content = additional_content
        else:
            content = {}

        content['activity'] = get_activity_html(request=request,
                                                activity=activity)
        super(ActivityResponse, self).__init__(content=content, status=status,
                                               **kwargs)



from __future__ import unicode_literals

from django.utils.translation import ugettext as _


class EnumCheck():
    """Adds a method to check if the enumeration value is a valid value.  If
    it is, then this returns the value.
    """

    @classmethod
    def check(cls, str_check):
        """Checks to see if a string is a value enum value. If yes, return the
        enum value.  Otherwise, return None.
        """
        if not str_check:
            return None

        str_check = str_check.upper()

        for s in cls.CHOICES:
            if s[0] == str_check:
                return s[0]

        return None

    @classmethod
    def get_display(cls, val):
        """Gets the display, human readable version of the enum value."""
        if not val:
            return None

        val = val.upper()

        for s in cls.CHOICES:
            if s[0] == val:
                return s[1]

        return None


class Source(EnumCheck):
    """The activity source represents what generated the activity.

    * SYSTEM - some system update performed on an object (i.e. a field update)
    * USER - a user is the source
    """
    SYSTEM = 'SYSTEM'
    USER = 'USER'
    CHOICES = ((SYSTEM, _('System')),
               (USER, _('User')),
               )


class Action(EnumCheck):
    """The past tense action choices for a activity.

    * COMMENTED - a user comment
    * CREATED - a created action
    * DELETED - a delete action
    * UPDATED - an update action
    """
    COMMENTED = 'COMMENTED'
    CREATED = 'CREATED'
    DELETED = 'DELETED'
    UPDATED = 'UPDATED'
    CHOICES = ((COMMENTED, _('Commented')),
               (CREATED, _('Created')),
               (DELETED, _('Deleted')),
               (UPDATED, _('Updated'))
               )


class Privacy(object):
    """Privacy for a activity."""
    PUBLIC = 'PUBLIC'  # everyone can see
    PRIVATE = 'PRIVATE'  # only created user can see
    CUSTOM = 'CUSTOM'  # a custom set of object has visibility
    CHOICES = ((PUBLIC, _('Public - everyone can see')),
               (PRIVATE, _('Private - only created user can see')),
               (CUSTOM, _('Custom - users must be granted visibility')),
               )



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.db.models.query_utils import Q
from django_core.db.models import CommonManager

from .constants import Action
from .constants import Privacy
from .constants import Source


class ActivityManager(CommonManager):
    """Manager for activities."""

    def create(self, created_user, text=None, about=None,
               source=Source.SYSTEM, action=Action.CREATED,
               ensure_for_objs=None, exclude_objs=None, **kwargs):
        """Creates a activity.

        :param created_user: the user document who created the activity.
        :param text: the text of the activity
        :param obj: the document this activity is for.
        :param source: the source of the activities. Can be one of
            contant.Source values.
        :param action: the action verb.  In this instance, it should almost
            always be CREATED
        :param ensure_for_objs: list of object to ensure will receive the
            activity.
        :param exclude_objs: exclude these objects from receiving the
            activity.
        :return: if activity is successfully added this returns True.
            Doesn't return entire object because the could potentially be a ton
            of activities and I won't want to return all of them.

        """
        if about is not None:
            kwargs['about'] = about

        n = super(ActivityManager, self).create(
            text=text.strip() if text else text,
            created_user=created_user,
            last_modified_user=created_user,
            source=source,
            action=action,
            **kwargs
        )

        for_objs = set([about])

        if ensure_for_objs:
            if not isinstance(ensure_for_objs, (list, tuple, set)):
                ensure_for_objs = set([ensure_for_objs])

            for_objs.update(ensure_for_objs)

        # Remove any objects that should be excluded from the activity
        if exclude_objs:
            for obj in exclude_objs:
                if obj in for_objs:
                    for_objs.remove(obj)

        for_model = self.model._get_many_to_many_model(field_name='for_objs')

        # This is a bit annoying.  So I have to loop through these 1 by 1
        # instead of using the bulk_create from the object manager because the
        # bulk_create statement doesn't return primary keys which is needed for
        # for_objs related manager add function call. See:
        # https://code.djangoproject.com/ticket/19527
        for_objs = [for_model.objects.get_or_create_generic(
                                                        content_object=obj)[0]
                    for obj in for_objs if obj is not None]

        n.for_objs.add(*for_objs)
        return n

    def get_for_user(self, user, **kwargs):
        """Gets activities for a user.

        :param user: the user to get the activities for

        """
        return self.get_for_object(obj=user, **kwargs)

    def get_for_object(self, obj, for_user=None, **kwargs):
        """Gets activities for a specific object.

        If ``for_user`` is provided, this will return all activities for the
        object that are public and will also return all private activities
        that the ``for_user`` has access to.

        Example use case, "Jane" is on "John's" profile page full of
        activities.  Jane is the for_user and accessing John's
        activities. John is the ``obj``. Jane will be able to see all public
        activities as well as any private activities she specifically
        has be granted access to see.

        :param obj: the object the activities are for
        :param for_user: only activities that this user can see
        :param kwargs: any key value pair fields that are on the model.

        """
        content_type = ContentType.objects.get_for_model(obj)
        queryset = self.filter(for_objs__content_type=content_type,
                               for_objs__object_id=obj.id,
                               **kwargs)

        if for_user is None or not for_user.is_authenticated():
            return queryset.filter(privacy=Privacy.PUBLIC).distinct()

        if for_user and for_user == obj:
            return queryset

        user_content_type = ContentType.objects.get_for_model(for_user)
        return queryset.filter(Q(created_user=for_user) |
                               Q(privacy=Privacy.PUBLIC) |
                               Q(privacy=Privacy.CUSTOM,
                                 for_objs__content_type=user_content_type,
                                 for_objs__object_id=for_user.id)).distinct()


class ActivityReplyManager(CommonManager):
    """Manager for activity replies."""

    def create(self, created_user, activity, text, reply_to=None,
               **kwargs):
        """Creates a new activity reply.

        :param created: the user creating the reply.
        :param activity: the activity this reply is about.
        :param text: the text of the activity.
        :param reply_to: the reply this reply is about.
        """
        return super(ActivityReplyManager, self).create(
            created_user=created_user,
            text=text,
            activity=activity,
            reply_to=reply_to,
            **kwargs
        )

    def get_by_activity(self, activity):
        """Gets all objects for a activity object."""
        try:
            return self.get(activity=activity)
        except self.model.DoesNotExist:
            return None

    def get_by_activity_id(self, activity_id):
        """Gets all objects by activity id."""
        try:
            return self.get(activity_id=activity_id)
        except self.model.DoesNotExist:
            return None



from __future__ import unicode_literals

import os

from setuptools import find_packages
from setuptools import setup


classifiers = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
]

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

setup(
    name='django-activities',
    version='1.1.0',
    description='Activities app for django',
    author='Troy Grosfield',
    maintainer='Troy Grosfield',
    url='https://github.com/infoagetech/django-activities',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    setup_requires=[
        'django >= 1.7',
        'django-core >= 1.4.0'
    ],
    classifiers=classifiers
)
